# Working with DB

In this demo we'll learn to write some DB queries using [Prisma](https://www.prisma.io/docs). This example relies on Postgresql database running inside Docker container.

## Getting started

### 1. Make sure your DB is running

First, you need to have a `.env` file with valid connection string - `.env.example` is here to help you.

Then you can run the DB:

```sh
docker run --detach -e POSTGRES_USER=user -e POSTGRES_PASSWORD=password -e POSTGRES_DB=database -p 5432:5432 --name pb138-seminar-08-database postgres:latest
```

### 2. Install dependencies & get familiar w/ the project

```
npm i
```

Look into `package.json` to see what scripts are available, get familiar with the data model in `prisma/schema.prisma`.

Run the following command to seed the DB:

```
npm run seed
```

Now, your DB should be ready for our demo! Check it out in the Prisma Studio:

```
npx prisma studio
```

## Exercise

1. Write function for creating a library member in `src/repositories/member/`.
2. Implement function for reading all loans in `src/repositories/loans`. This functions takes optional parameters for sorting based on creation date (`loanDate`) and for filtering based on (`returnDate`) - loan is active if `returnDate` is null.
3. Implement function for creating a loan in `src/repositories/loans`. You might want to use transaction, where you will obtain IDs of member and book, check if the book isn't loaned and then create a new loan.
4. Prisma can do more than just basic CRUD operations. In `src/repositories/book/read.ts` implement a simple function which returns sum of pages of all books we have.
